/*
 * (c) 2014 Rechenraum e.U. (Simon Flöry; simon.floery@rechenraum.com)
 *
 * This file is part of project oe3d (www.oe3d.at).
 *
 * oe3d is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * oe3d is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with oe3d. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _HORIZON_H
#define _HORIZON_H

#include <vector>
#include <string>

#include <libnova/ln_types.h>

#include "sunnig.h"
#include "Observer.h"

/*! \file Horizon data structure.
 * 
 */
class Horizon
{
public:

	/* documented in source code */
    Horizon();
	
    /* documented in source code */
    bool load(const std::string&, Observer&);

    /* documented in source code */
    unsigned int az2index(const GeCore::GCReal) const;
    GeCore::GCReal index2az(const unsigned int) const;

    /* documented in source code */
    GeCore::GCReal getAzMin() const;
    GeCore::GCReal getAzMax() const;
    const std::vector< GeCore::GCReal >& getAltitude() const;

protected:

    /*! Start of azimuth range of horizon IN DEGREES. We ensure
     * m_azMin < m_azMax, but NOT m_azMin > 0 or (m_azMax - m_azMin) < 360.
     */
    GeCore::GCReal m_azMin;

    /*! End of azimuth range of horizon IN DEGREES.
     */
    GeCore::GCReal m_azMax;

    /*! Azimuth range of horizon IN DEGREES */
    GeCore::GCReal m_azRange;

    /*! array storing the maximal altitude. The altitude is stored as angle in
     * DEGREES from the fundamental plane.
     */
    std::vector< GeCore::GCReal > m_altitude;
};

#endif // _HORIZON_H
