/*
 * (c) 2014 Rechenraum e.U. (Simon Flöry; simon.floery@rechenraum.com)
 *
 * This file is part of project oe3d (www.oe3d.at).
 *
 * oe3d is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * oe3d is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with oe3d. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef EVALUATOR_LABELEDSUNPATH_H
#define EVALUATOR_LABELEDSUNPATH_H

/* forward declaration */
template< typename T > class EvaluatorLabeledSunpath;

#include "../sunnig.h"
#include "../Observer.h"
#include "Evaluator.h"
#include "../LabeledSunpath.h"

//------------------------------------------------------------------------------

/******************************************************************************
 *
 * class EvaluatorLabeledSunpath< T >
 *
 ******************************************************************************/

//------------------------------------------------------------------------------
/*! \brief Abstract base class for evaluating classes working on labeled sunpaths
 *
 * This class is based on Evaluator< T > for convenience purposes that stores
 * a const pointer to a label sunpath object.
 */
template< typename T > class EvaluatorLabeledSunpath : public Evaluator< T >
{
public:
    //--------------------------------------------------------------------------

    /*! \brief Constructor
     *
     * The constructor stores the const reference to the options and a const
     * pointer to the labeled sunpath.
     *
     * \param options Const reference to the options.
     */
    EvaluatorLabeledSunpath(const Observer& obs)
        : Evaluator< T >(obs), m_sunpath(NULL)
    {

    }

    //--------------------------------------------------------------------------

    /*! \brief Load labeled sunpath
     *
     * Load the labeled sunpath.
     *
     * \param sunpath Const pointer to a labeled sunpath object. We will do a
     *      shallow copy.
     *
     * \return True on success, false otherwise.
     */
    virtual bool load(const LabeledSunpath* sunpath) = 0;

    //--------------------------------------------------------------------------

protected:
    /*! Pointer to labeled sunpath */
    const LabeledSunpath* m_sunpath;
};

//------------------------------------------------------------------------------

#endif // EVALUATOR_LABELEDSUNPATH_H
