/*
 * (c) 2014 Rechenraum e.U. (Simon Flöry; simon.floery@rechenraum.com)
 *
 * This file is part of project oe3d (www.oe3d.at).
 *
 * oe3d is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * oe3d is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with oe3d. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef EVALUATOR_SUNSHINE_DURATION_H
#define EVALUATOR_SUNSHINE_DURATION_H

/* forward declaration */
class EvaluatorSunshineDuration;

#include "EvaluatorLabeledSunpath.h"

//------------------------------------------------------------------------------

/******************************************************************************
 *
 * class EvaluatorSunshineDuration
 *
 ******************************************************************************/

//------------------------------------------------------------------------------

/*! \brief
 *
 * Attention: A Julian day starts at noon!
 */
class EvaluatorSunshineDuration : public EvaluatorLabeledSunpath< GeCore::GCReal >
{
public:
    /* calls to members declared only in a base template class are not resolved
     * when they are encountered, as their might be specializations that are
     * found later. So we end up with compute and computePlain unknown, unless
     * we explicitely bring them into scope with the following using
     * declarations. See
     *      http://womble.decadent.org.uk/c++/template-faq.html#base-lookup
     * for more information.
     */
    using Evaluator< GeCore::GCReal >::compute;
    using Evaluator< GeCore::GCReal >::computePlain;

    /* documented in source code */
    EvaluatorSunshineDuration(const Observer&);

    /* documented in source code */
    bool load(const LabeledSunpath*);

    /* documented in source code */
    GeCore::GCReal compute(double, double);

    /* documented in source code */
    GeCore::GCReal computePlain(double, double);

    /* documented in source code */
    static GeCore::GCReal toHours(double);

protected:

};

#endif // EVALUATOR_SUNSHINE_DURATION_H
