/*
 * (c) 2014 Rechenraum e.U. (Simon Flöry; simon.floery@rechenraum.com)
 *
 * This file is part of project oe3d (www.oe3d.at).
 *
 * oe3d is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * oe3d is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with oe3d. If not, see <http://www.gnu.org/licenses/>.
 */

#include "../sunnig.h"

namespace GeCore
{

template< typename ScalarT > class GC3DT
{
public:
	GC3DT()
	{

	}

	GC3DT(ScalarT _x, ScalarT _y, ScalarT _z)
	{
		x = _x;
		y = _y;
		z = _z;
	}

    ScalarT dot(const GC3DT& that)
    {
        return x * that.x + y * that.y + z * that.z;
    }

	ScalarT x;
	ScalarT y;
	ScalarT z;
};

template< typename ScalarT > class GC2DT
{
public:
    GC2DT()
    {

    }

    GC2DT(ScalarT _x, ScalarT _y)
    {
        x = _x;
        y = _y;
    }

    GC2DT operator+(const GC2DT& that) const
    {
        return GC2DT(x + that.x, y + that.y);
    }

    GC2DT operator-(const GC2DT& that) const
    {
        return GC2DT(x - that.x, y - that.y);
    }

    GC2DT operator*(const GCReal factor) const
    {
        return GC2DT(x*factor, y*factor);
    }

    ScalarT x;
    ScalarT y;
};

typedef GC3DT< GCReal > GC3D;
typedef GC2DT< GCReal > GC2D;

//------------------------------------------------------------------------------

/*! \brief Compute signed area of the triangle spanned by the given three 2D points
 *
 *
 * \param a First point
 * \param b Second point
 * \param c Third point
 *
 * \return Signed area.
 */
template< class pointT > GeCore::GCReal computeSignedArea(const pointT& a, const pointT& b, const pointT& c)
{
    return 0.5 * ((b.x - a.x)*(c.y - a.y) - (b.y - a.y)*(c.x - a.x));
}

//------------------------------------------------------------------------------

}
