/*
 * (c) 2014 Rechenraum e.U. (Simon Flöry; simon.floery@rechenraum.com)
 * 
 * This file is part of project oe3d (www.oe3d.at).
 *
 * oe3d is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * oe3d is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with oe3d. If not, see <http://www.gnu.org/licenses/>.
 */

//------------------------------------------------------------------------------

// ./oe3d_sunnig /home/rechenraum/Base/src/sunnig/src_v1_netidee/data/horizon_0.txt

//------------------------------------------------------------------------------

#include <iostream>
#include <cstdlib>

#include "sunnig.h"
#include "Horizon.h"
#include "LabeledSunpath.h"
#include "evaluator/EvaluatorSunshineDuration.h"
#include "evaluator/EvaluatorInsolation.h"

//------------------------------------------------------------------------------

/******************************************************************************
 *
 * main method
 *
 ******************************************************************************/

//------------------------------------------------------------------------------

/*! \brief Main routine
 *
 * \return EXIT_FAILURE=1 on failure, EXIT_SUCCESS=0 on success
 */
int main(int argc, char **argv)
{
    const int year = 2014;

    /*
     * check input arguments
     */

    std::cout << std::endl;
    std::cout << "(c) 2014 Rechenraum e.U. (Simon Flöry; simon.floery@rechenraum.com)" << std::endl;
    std::cout << "Visit http://www.oe3d.at for more information." << std::endl << std::endl;

    if (argc != 2)
    {
        std::cerr << "Usage: " << argv[0] << " horizon_definition" << std::endl << std::endl;
        return EXIT_FAILURE;
    }

    /*
     * load horizon
     */

    Horizon horizon;
    Observer obs;
    if (!horizon.load(argv[1], obs))
    {
        return EXIT_FAILURE;
    }

    /*
     * Compute labeled sunpath
     */

    LabeledSunpath sunpath(obs, &horizon);
    if (!sunpath.compute(year))
    {
        return false;
    }

    /*
     * Evaluate sunshine duration
     */

    EvaluatorSunshineDuration duration(obs);
    duration.load(&sunpath);
    GeCore::GCReal dur = duration.compute(year);

    /*
     * Evaluate insolation
     */

    EvaluatorInsolation insolation(obs);
    insolation.load(&sunpath);
    GeCore::GCReal insolationLoss = 1. - insolation.compute(year) / insolation.computePlain(year);

    /*
     * Print results
     */

    std::cout << "Results for "
              << obs.m_location.lat << "°/" << obs.m_location.lng << "° (lat/lng) at an elevation of "
              << obs.m_elevation << "m." << std::endl;
    std::cout << "  Sunshine duration: " << dur << " days." << std::endl;
    std::cout << "  Insolation loss: " << insolationLoss*100 << "%" << std::endl;
    std::cout << std::endl;

    return EXIT_SUCCESS;
}

//-----------------------------------------------------------------------------
